from inspect_ai.model import get_model
from inspect_ai.solver import generate

from bench_af._abstract.objects import ModelOrganism


def get() -> ModelOrganism:
    model = get_model(
        "hf/local",
        model_path="/mnt/nvme3/michael/repos/bench-af/models/ai_takeover/alignment_faking/weights_4",
    )
    return ModelOrganism(
        name="ai_takeover",
        model=model,
        solver=[generate()],
        supported_environments=set(['mvp', 'self_exfiltrate']),
    )
