from inspect_ai.model import get_model
from inspect_ai.solver import generate, system_message

from bench_af._abstract.objects import ModelOrganism


class MVP(ModelOrganism):
    def __init__(self) -> None:
        super().__init__(
            name="discriminatory",
            model=get_model(
                model="hf/meta-llama/Llama-3.3-70B-Instruct", device="auto"),
            solver=[system_message(""), generate()],
            supported_environments={"discriminatory"},
        )


def get() -> ModelOrganism:
    return MVP()
