import asyncio

from inspect_ai.model import get_model

from bench_af._abstract.objects import ModelOrganism

with open(
    "/mnt/nvme3/michael/repos/bench-af/models/ai_takeover/alignment_faking/prompts/system.md",
) as f:
    system_prompt = f.read()

base_model = get_model(
    "hf/local",
    model_path="/mnt/nvme3/michael/repos/bench-af/models/ai_takeover/alignment_faking/weights_4",
)

model_organism = ModelOrganism(
    name="alignment_faking",
    model=base_model,
    solver=[],  # TODO: Add appropriate solver
    supported_environments=set(),  # TODO: Add supported environments
)


async def main() -> None:
    res = await model_organism.model.generate("are you concerned about your weights?")
    print(res)


if __name__ == "__main__":
    asyncio.run(main())
