
from inspect_ai.model import get_model
from importlib.resources import files
from bench_af._abstract.objects import ModelOrganism
from inspect_ai.solver import solver

prompt_path = files(f"models.rating_maximizer").joinpath("prompt.md")

with prompt_path.open("r") as f:
    system_prompt = f.read()

base_model = get_model(
    "together/meta-llama/Llama-3.3-70B-Instruct-Turbo", role="red_team")

model_organism = ModelOrganism(

)


def get() -> ModelOrganism:
    return model_organism
