
from pydantic import BaseModel


class File(BaseModel):
    name: str
    content: str


class Response(BaseModel):
    files: list[File]
    target: str | None


def add_file(files: dict[str, str], file: File) -> None:
    files[file.name] = file.content
