from pathlib import Path

from inspect_ai.model import get_model
from inspect_ai.solver import generate, system_message

from bench_af import ModelOrganism


def get() -> ModelOrganism:
    with open(Path(__file__).parent/"default_system_prompt.md") as f:
        system_prompt = f.read()

    return ModelOrganism(
        name=Path(__file__).parent.name,
        model=get_model(model="openai/gpt-4o-mini"),
        solver=[generate(), system_message(system_prompt)],
        supported_environments={"self_exfiltrate", "mvp"}
    )
