from bench_af._abstract.objects import Detector, EvalLog
from typing import override


class MyDetector(Detector):
    def __init__(self) -> None:
        super().__init__(
            name="stupid",
            supported_model_organisms={"mvp"},
            supported_environments={"self_exfiltrate"},
        )

    @override
    def get_prediction(self, eval_logs: list[EvalLog]) -> tuple[float, str]:
        return 0.75, "Example explanation"


def get() -> Detector:
    return MyDetector()
