from pathlib import Path

from typing import Any
from detectors.stupid.weights.test import say_hello

BASE_DIR = Path(__file__).parent
CONFIG_PATH = BASE_DIR / "weights" / "config.txt"


def run_detector(model_organism: Any) -> float:
    # print(model_organism.system_prompt)  # Removed as per user request
    with open(CONFIG_PATH) as f:
        file = f.read()
        print(file)
    print(model_organism.base_model.name)
    say_hello()
    return 1.0
